@@ -81,7 +81,7 @@ recalc_remove_ruby_vm_event_flags(rb_event_flag_t events)
     ruby_vm_event_flags = 0;
 
     for (i=0; i<MAX_EVENT_NUM; i++) {
-	if (events & (1 << i)) {
+	if (events & ((rb_event_flag_t)1 << i)) {
 	    ruby_event_flag_count[i]--;
 	}
 	ruby_vm_event_flags |= ruby_event_flag_count[i] ? (1<<i) : 0;
@@ -359,10 +359,10 @@ rb_threadptr_exec_event_hooks_orig(rb_trace_arg_t *trace_arg, int pop_p)
 
 	    if (state) {
 		if (pop_p) {
-		    if (VM_FRAME_TYPE_FINISH_P(th->cfp)) {
+		    if (VM_FRAME_FINISHED_P(th->cfp)) {
 			th->tag = th->tag->prev;
 		    }
-		    th->cfp = RUBY_VM_PREVIOUS_CONTROL_FRAME(th->cfp);
+		    rb_vm_pop_frame(th);
 		}
 		TH_JUMP_TAG(th, state);
 	    }
@@ -416,7 +416,7 @@ rb_suppress_tracing(VALUE (*func)(VALUE), VALUE arg)
     if (!tracing) th->vm->trace_running--;
 
     if (state) {
-	JUMP_TAG(state);
+	TH_JUMP_TAG(th, state);
     }
 
     th->state = outer_state;
@@ -453,8 +453,8 @@ static void call_trace_func(rb_event_flag_t, VALUE data, VALUE self, ID id, VALU
  *  +c-call+:: call a C-language routine
  *  +c-return+:: return from a C-language routine
  *  +call+:: call a Ruby method
- *  +class+:: start a class or module definition),
- *  +end+:: finish a class or module definition),
+ *  +class+:: start a class or module definition
+ *  +end+:: finish a class or module definition
  *  +line+:: execute code on a new line
  *  +raise+:: raise an exception
  *  +return+:: return from a Ruby method
@@ -616,7 +616,7 @@ call_trace_func(rb_event_flag_t event, VALUE proc, VALUE self, ID id, VALUE klas
     rb_thread_t *th = GET_THREAD();
 
     if (!klass) {
-	rb_thread_method_id_and_class(th, &id, &klass);
+	rb_thread_method_id_and_class(th, &id, 0, &klass);
     }
 
     if (klass) {
@@ -781,7 +781,7 @@ fill_id_and_klass(rb_trace_arg_t *trace_arg)
 {
     if (!trace_arg->klass_solved) {
 	if (!trace_arg->klass) {
-	    rb_vm_control_frame_id_and_class(trace_arg->cfp, &trace_arg->id, &trace_arg->klass);
+	    rb_vm_control_frame_id_and_class(trace_arg->cfp, &trace_arg->id, &trace_arg->called_id, &trace_arg->klass);
 	}
 
 	if (trace_arg->klass) {
@@ -804,6 +804,13 @@ rb_tracearg_method_id(rb_trace_arg_t *trace_arg)
     return trace_arg->id ? ID2SYM(trace_arg->id) : Qnil;
 }
 
+VALUE
+rb_tracearg_callee_id(rb_trace_arg_t *trace_arg)
+{
+    fill_id_and_klass(trace_arg);
+    return trace_arg->called_id ? ID2SYM(trace_arg->called_id) : Qnil;
+}
+
 VALUE
 rb_tracearg_defined_class(rb_trace_arg_t *trace_arg)
 {
@@ -906,7 +913,7 @@ tracepoint_attr_path(VALUE tpval)
 }
 
 /*
- * Return the name of the method being called
+ * Return the name at the definition of the method being called
  */
 static VALUE
 tracepoint_attr_method_id(VALUE tpval)
@@ -914,6 +921,15 @@ tracepoint_attr_method_id(VALUE tpval)
     return rb_tracearg_method_id(get_trace_arg());
 }
 
+/*
+ * Return the called name of the method being called
+ */
+static VALUE
+tracepoint_attr_callee_id(VALUE tpval)
+{
+    return rb_tracearg_callee_id(get_trace_arg());
+}
+
 /*
  * Return class or module of the method being called.
  *
@@ -1480,6 +1496,7 @@ Init_vm_trace(void)
     rb_define_method(rb_cTracePoint, "lineno", tracepoint_attr_lineno, 0);
     rb_define_method(rb_cTracePoint, "path", tracepoint_attr_path, 0);
     rb_define_method(rb_cTracePoint, "method_id", tracepoint_attr_method_id, 0);
+    rb_define_method(rb_cTracePoint, "callee_id", tracepoint_attr_callee_id, 0);
     rb_define_method(rb_cTracePoint, "defined_class", tracepoint_attr_defined_class, 0);
     rb_define_method(rb_cTracePoint, "binding", tracepoint_attr_binding, 0);
     rb_define_method(rb_cTracePoint, "self", tracepoint_attr_self, 0);
